@@ -7,8 +7,10 @@
 */
 
 #include "filemetadataprovider.h"
+#include "filemetadatautil_p.h"
 #include "filefetchjob.h"
 
+#include <Baloo/IndexerConfig>
 #include <KFileMetaData/PropertyInfo>
 #include <KFormat>
 #include <KLocalizedString>
@@ -29,51 +31,6 @@ using namespace Baloo;
 
 namespace
 {
-QVariant intersect(const QVariant &v1, const QVariant &v2)
-{
-    if (!v1.isValid() || !v2.isValid()) {
-        return {};
-    }
-
-    // List and String
-    if (v1.type() == QVariant::StringList && v2.type() == QVariant::String) {
-        QStringList list = v1.toStringList();
-        QString str = v2.toString();
-
-        if (!list.contains(str)) {
-            list << str;
-        }
-
-        return QVariant(list);
-    }
-
-    // String and List
-    if (v1.type() == QVariant::String && v2.type() == QVariant::StringList) {
-        QStringList list = v2.toStringList();
-        QString str = v1.toString();
-
-        if (!list.contains(str)) {
-            list << str;
-        }
-
-        return QVariant(list);
-    }
-
-    // List and List
-    if (v1.type() == QVariant::StringList && v2.type() == QVariant::StringList) {
-        QSet<QString> s1(v1.toStringList().cbegin(), v1.toStringList().cend());
-        QSet<QString> s2(v2.toStringList().cbegin(), v2.toStringList().cend());
-
-        return QVariant(s1.intersect(s2).values());
-    }
-
-    if (v1 == v2) {
-        return v1;
-    }
-
-    return {};
-}
-
 /**
  * The standard QMap::unite will contain the key multiple times if both \p v1 and \p v2
  * contain the same key.
@@ -92,30 +49,117 @@ QVariantMap unite(const QVariantMap &v1, const QVariantMap &v2)
 
     return v;
 }
-} // anonymous namespace
 
-void FileMetaDataProvider::totalPropertyAndInsert(const QString &prop, const QList<QVariantMap> &resources, QSet<QString> &allProperties)
+/**
+* @return The number of files and hidden files for the directory path.
+*/
+QPair<int, int> subDirectoriesCount(const QString &path)
 {
-    if (allProperties.contains(prop)) {
-        int total = 0;
-        for (const QVariantMap &map : resources) {
-            QVariantMap::const_iterator it = map.constFind(prop);
-            if (it == map.constEnd()) {
-                total = 0;
-                break;
+#ifdef Q_OS_WIN
+    QDir dir(path);
+    int count = dir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::System).count();
+    int hiddenCount = dir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::System | QDir::Hidden).count();
+    return QPair<int, int>(count, hiddenCount);
+#else
+    // Taken from kdelibs/kio/kio/kdirmodel.cpp
+    // SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
+
+    int count = -1;
+    int hiddenCount = -1;
+    DIR *dir = ::opendir(QFile::encodeName(path).constData());
+    if (dir) {
+        count = 0;
+        hiddenCount = 0;
+        struct dirent *dirEntry = nullptr;
+        while ((dirEntry = ::readdir(dir))) { // krazy:exclude=syscalls
+            if (dirEntry->d_name[0] == '.') {
+                if (dirEntry->d_name[1] == '\0') {
+                    // Skip "."
+                    continue;
+                }
+                if (dirEntry->d_name[1] == '.' && dirEntry->d_name[2] == '\0') {
+                    // Skip ".."
+                    continue;
+                }
+                // hidden files
+                hiddenCount++;
             } else {
-                total += it.value().toInt();
+                ++count;
             }
         }
-
-        if (total) {
-            m_data.insert(prop, QVariant(total));
-        }
-        allProperties.remove(prop);
+        ::closedir(dir);
     }
+    return QPair<int, int>(count, hiddenCount);
+#endif
 }
 
-void FileMetaDataProvider::slotFileFetchFinished(KJob *job)
+/**
+ * Fill \p data with properties can be derived from others
+ */
+void extractDerivedProperties(QVariantMap &data)
+{
+    const auto width = data.value(QStringLiteral("width"));
+    const auto height = data.value(QStringLiteral("height"));
+    if (!width.isNull() && !height.isNull()) {
+        data.insert(QStringLiteral("dimensions"), i18nc("width × height", "%1 × %2", width.toInt(), height.toInt()));
+    }
+
+    bool okLatitude;
+    const auto gpsLatitude = data.value(QStringLiteral("photoGpsLatitude")).toFloat(&okLatitude);
+    bool okLongitude;
+    const auto gpsLongitude = data.value(QStringLiteral("photoGpsLongitude")).toFloat(&okLongitude);
+
+    if (okLatitude && okLongitude) {
+        data.insert(QStringLiteral("gpsLocation"), QVariant::fromValue(QPair<float, float>(gpsLatitude, gpsLongitude)));
+    }
+}
+} // anonymous namespace
+
+class Q_DECL_HIDDEN Baloo::FileMetaDataProviderPrivate : public QObject
+{
+public:
+    FileMetaDataProviderPrivate(FileMetaDataProvider *parent)
+        : QObject(parent)
+        , m_parent(parent)
+        , m_readOnly(false)
+    {
+    }
+
+    ~FileMetaDataProviderPrivate()
+    {
+    }
+
+    void insertEditableData();
+
+    void processFileItems();
+
+    void setFileItem();
+    void setFileItems();
+
+    /**
+     * Insert basic data of a single file
+     */
+    void insertSingleFileBasicData();
+
+    /**
+     * Insert basic data of a list of files
+     */
+    void insertFilesListBasicData();
+
+    FileMetaDataProvider *m_parent;
+
+    bool m_readOnly;
+
+    QList<KFileItem> m_fileItems;
+
+    QVariantMap m_data;
+    Baloo::IndexerConfig m_config;
+
+public Q_SLOTS:
+    void slotFileFetchFinished(KJob *job);
+};
+
+void FileMetaDataProviderPrivate::slotFileFetchFinished(KJob *job)
 {
     auto fetchJob = static_cast<FileFetchJob *>(job);
     QList<QVariantMap> files = fetchJob->data();
@@ -123,47 +167,31 @@ void FileMetaDataProvider::slotFileFetchFinished(KJob *job)
     Q_ASSERT(!files.isEmpty());
 
     if (files.size() > 1) {
-        insertCommonData(files);
+        Baloo::Private::mergeCommonData(m_data, files);
     } else {
         m_data = unite(m_data, files.first());
-
-        const auto width = m_data.value(QStringLiteral("width"));
-        const auto height = m_data.value(QStringLiteral("height"));
-        if ((width.type() == QVariant::Double || width.type() == QVariant::Int) && (height.type() == QVariant::Double || height.type() == QVariant::Int)) {
-            m_data.insert(QStringLiteral("dimensions"), i18nc("width × height", "%1 × %2", width.toInt(), height.toInt()));
-        }
-
-        bool okLatitude;
-        const auto gpsLatitude = m_data.value(QStringLiteral("photoGpsLatitude")).toFloat(&okLatitude);
-        bool okLongitude;
-        const auto gpsLongitude = m_data.value(QStringLiteral("photoGpsLongitude")).toFloat(&okLongitude);
-
-        if (okLatitude && okLongitude) {
-            m_data.insert(QStringLiteral("gpsLocation"), QVariant::fromValue(QPair<float, float>(gpsLatitude, gpsLongitude)));
-        }
     }
+    extractDerivedProperties(m_data);
     m_readOnly = !fetchJob->canEditAll();
 
     insertEditableData();
-    Q_EMIT loadingFinished();
+    Q_EMIT m_parent->loadingFinished();
 }
 
-void FileMetaDataProvider::insertSingleFileBasicData()
+void FileMetaDataProviderPrivate::insertSingleFileBasicData()
 {
     // TODO: Handle case if remote URLs are used properly. isDir() does
     // not work, the modification date needs also to be adjusted...
-    Q_ASSERT(m_fileItems.count() <= 1);
-    if (m_fileItems.count() == 1) {
+    Q_ASSERT(m_fileItems.count() == 1);
+    {
         const KFileItem &item = m_fileItems.first();
 
         KFormat format;
         if (item.isDir()) {
-            bool isSizeUnknown = !item.isLocalFile() || item.isSlow();
-            if (!isSizeUnknown) {
+            if (item.isLocalFile() && !item.isSlow()) {
                 const QPair<int, int> counts = subDirectoriesCount(item.url().path());
                 const int count = counts.first;
-                isSizeUnknown = count == -1;
-                if (!isSizeUnknown) {
+                if (count != -1) {
                     QString itemCountString = i18ncp("@item:intable", "%1 item", "%1 items", count);
                     m_data.insert(QStringLiteral("kfileitem#size"), itemCountString);
 
@@ -238,7 +266,7 @@ void FileMetaDataProvider::insertSingleFileBasicData()
     }
 }
 
-void FileMetaDataProvider::insertFilesListBasicData()
+void FileMetaDataProviderPrivate::insertFilesListBasicData()
 {
     // If all directories
     Q_ASSERT(m_fileItems.count() > 1);
@@ -253,16 +281,13 @@ void FileMetaDataProvider::insertFilesListBasicData()
     if (allDirectories) {
         int count = 0;
         int hiddenCount = 0;
-        bool isSizeKnown = true;
         for (const KFileItem &item : std::as_const(m_fileItems)) {
-            isSizeKnown = item.isLocalFile() && !item.isSlow();
-            if (!isSizeKnown) {
+            if (!item.isLocalFile() || item.isSlow()) {
                 return;
             }
             const QPair<int, int> counts = subDirectoriesCount(item.url().path());
             const int subcount = counts.first;
-            isSizeKnown = subcount != -1;
-            if (!isSizeKnown) {
+            if (subcount == -1) {
                 return;
             }
             count += subcount;
@@ -289,7 +314,7 @@ void FileMetaDataProvider::insertFilesListBasicData()
     }
 }
 
-void FileMetaDataProvider::insertEditableData()
+void FileMetaDataProviderPrivate::insertEditableData()
 {
     if (!m_readOnly) {
         if (!m_data.contains(QStringLiteral("tags"))) {
@@ -304,99 +329,28 @@ void FileMetaDataProvider::insertEditableData()
     }
 }
 
-void FileMetaDataProvider::insertCommonData(const QList<QVariantMap> &files)
-{
-    //
-    // Only report the stuff that is common to all the files
-    //
-    QSet<QString> allProperties;
-    QList<QVariantMap> propertyList;
-    for (const QVariantMap &fileData : files) {
-        propertyList << fileData;
-        auto uniqueValues = fileData.keys();
-        uniqueValues.erase(std::unique(uniqueValues.begin(), uniqueValues.end()), uniqueValues.end());
-        allProperties += QSet<QString>(uniqueValues.begin(), uniqueValues.end());
-    }
-
-    // Special handling for certain properties
-    totalPropertyAndInsert(QStringLiteral("duration"), propertyList, allProperties);
-    totalPropertyAndInsert(QStringLiteral("characterCount"), propertyList, allProperties);
-    totalPropertyAndInsert(QStringLiteral("wordCount"), propertyList, allProperties);
-    totalPropertyAndInsert(QStringLiteral("lineCount"), propertyList, allProperties);
-
-    for (const QString &propUri : std::as_const(allProperties)) {
-        for (const QVariantMap &map : std::as_const(propertyList)) {
-            QVariantMap::const_iterator it = map.find(propUri);
-            if (it == map.constEnd()) {
-                m_data.remove(propUri);
-                break;
-            }
-
-            QVariantMap::iterator dit = m_data.find(it.key());
-            if (dit == m_data.end()) {
-                m_data.insert(propUri, it.value());
-            } else {
-                QVariant finalValue = intersect(it.value(), dit.value());
-                if (finalValue.isValid()) {
-                    m_data[propUri] = finalValue;
-                } else {
-                    m_data.remove(propUri);
-                    break;
-                }
-            }
-        }
-    }
-}
-
 FileMetaDataProvider::FileMetaDataProvider(QObject *parent)
     : QObject(parent)
-    , m_readOnly(false)
-    , m_realTimeIndexing(false)
+    , d(new FileMetaDataProviderPrivate(this))
 {
 }
 
 FileMetaDataProvider::~FileMetaDataProvider() = default;
 
-void FileMetaDataProvider::setFileItem()
+void FileMetaDataProviderPrivate::processFileItems()
 {
-    // There are 3 code paths -
+    // There are several code paths -
     // Remote file
     // Single local file -
     //   * Not Indexed
     //   * Indexed
     //
-    insertSingleFileBasicData();
-    const QUrl url = m_fileItems.first().targetUrl();
-    if (!url.isLocalFile() || m_fileItems.first().isSlow()) {
-        // FIXME - are extended attributes supported for remote files?
-        m_readOnly = true;
-        Q_EMIT loadingFinished();
-        return;
-    }
-
-    const QString filePath = url.toLocalFile();
-    FileFetchJob *job;
-
-    // Not indexed or only basic file indexing (no content)
-    if (!m_config.fileIndexingEnabled() || !m_config.shouldBeIndexed(filePath) || m_config.onlyBasicIndexing()) {
-        m_realTimeIndexing = true;
-
-        job = new FileFetchJob(QStringList{filePath}, true, FileFetchJob::UseRealtimeIndexing::Only, this);
-
-        // Fully indexed by Baloo
-    } else {
-        job = new FileFetchJob(QStringList{filePath}, true, FileFetchJob::UseRealtimeIndexing::Fallback, this);
-    }
-    connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
-    job->start();
-}
-
-void FileMetaDataProvider::setFileItems()
-{
     // Multiple Files -
     //   * Not Indexed
     //   * Indexed
 
+    bool singleFileMode = m_fileItems.size() <= 1;
+
     QStringList urls;
     urls.reserve(m_fileItems.size());
     // Only extract data from indexed files,
@@ -408,34 +362,49 @@ void FileMetaDataProvider::setFileItems()
         }
     }
 
-    insertFilesListBasicData();
+    if (singleFileMode) {
+        insertSingleFileBasicData();
+    } else {
+        insertFilesListBasicData();
+    }
+
     if (!urls.isEmpty()) {
         // Editing only if all URLs are local
         bool canEdit = (urls.size() == m_fileItems.size());
 
-        auto job = new FileFetchJob(urls, canEdit, FileFetchJob::UseRealtimeIndexing::Disabled, this);
-        connect(job, &FileFetchJob::finished, this, &FileMetaDataProvider::slotFileFetchFinished);
+        // Don't use indexing when we have multiple files
+        auto indexingMode = FileFetchJob::UseRealtimeIndexing::Disabled;
+
+        if (singleFileMode) {
+            // Fully indexed by Baloo
+            indexingMode = FileFetchJob::UseRealtimeIndexing::Fallback;
+
+            if (!m_config.fileIndexingEnabled() || !m_config.shouldBeIndexed(urls.first()) || m_config.onlyBasicIndexing()) {
+                // Not indexed or only basic file indexing (no content)
+                indexingMode = FileFetchJob::UseRealtimeIndexing::Only;
+            }
+        }
+
+        auto job = new FileFetchJob(urls, canEdit, indexingMode, this);
+        connect(job, &FileFetchJob::finished, this, &FileMetaDataProviderPrivate::slotFileFetchFinished);
         job->start();
 
     } else {
         // FIXME - are extended attributes supported for remote files?
         m_readOnly = true;
-        Q_EMIT loadingFinished();
+        Q_EMIT m_parent->loadingFinished();
     }
 }
 
 void FileMetaDataProvider::setItems(const KFileItemList &items)
 {
-    m_fileItems = items;
-    m_data.clear();
-    m_realTimeIndexing = false;
+    d->m_fileItems = items;
+    d->m_data.clear();
 
     if (items.isEmpty()) {
         Q_EMIT loadingFinished();
-    } else if (items.size() == 1) {
-        setFileItem();
     } else {
-        setFileItems();
+        d->processFileItems();
     }
 }
 
@@ -567,65 +536,22 @@ QString FileMetaDataProvider::group(const QString &label) const
 
 KFileItemList FileMetaDataProvider::items() const
 {
-    return m_fileItems;
+    return d->m_fileItems;
 }
 
 void FileMetaDataProvider::setReadOnly(bool readOnly)
 {
-    m_readOnly = readOnly;
+    d->m_readOnly = readOnly;
 }
 
 bool FileMetaDataProvider::isReadOnly() const
 {
-    return m_readOnly;
+    return d->m_readOnly;
 }
 
 QVariantMap FileMetaDataProvider::data() const
 {
-    return m_data;
+    return d->m_data;
 }
 
-QPair<int, int> FileMetaDataProvider::subDirectoriesCount(const QString &path)
-{
-#ifdef Q_OS_WIN
-    QDir dir(path);
-    int count = dir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::System).count();
-    int hiddenCount = dir.entryList(QDir::AllEntries | QDir::NoDotAndDotDot | QDir::System | QDir::Hidden).count();
-    return QPair<int, int>(count, hiddenCount);
-#else
-    // Taken from kdelibs/kio/kio/kdirmodel.cpp
-    // SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
-
-    int count = -1;
-    int hiddenCount = -1;
-    DIR *dir = ::opendir(QFile::encodeName(path).constData());
-    if (dir) {
-        count = 0;
-        hiddenCount = 0;
-        struct dirent *dirEntry = nullptr;
-        while ((dirEntry = ::readdir(dir))) { // krazy:exclude=syscalls
-            if (dirEntry->d_name[0] == '.') {
-                if (dirEntry->d_name[1] == '\0') {
-                    // Skip "."
-                    continue;
-                }
-                if (dirEntry->d_name[1] == '.' && dirEntry->d_name[2] == '\0') {
-                    // Skip ".."
-                    continue;
-                }
-                // hidden files
-                hiddenCount++;
-            } else {
-                ++count;
-            }
-        }
-        ::closedir(dir);
-    }
-    return QPair<int, int>(count, hiddenCount);
-#endif
-}
-
-bool FileMetaDataProvider::realTimeIndexing()
-{
-    return m_realTimeIndexing;
-}
+#include "moc_filemetadataprovider.cpp"
